#!/usr/bin/perl
#
################################################################################
#
# IPCop CopSpot AddOn Web-Iface
#
# Copyright (C) 2007 Olaf (weizen_42) Westrik
#
# This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version. 
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
#
# You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA 
#
#
# Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der GNU General Public License, wie von der Free Software Foundation verffentlicht, weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder (nach Ihrer Option) jeder spteren Version. 
#
# Die Verffentlichung dieses Programms erfolgt in der Hoffnung, da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 
#
# Sie sollten ein Exemplar der GNU General Public License zusammen mit diesem Programm erhalten haben. Falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
#
################################################################################
#
# For support post / read in http://www.ipcop-forum.de
# or contact the autor: weizen_42 at ipcop-forum dot de
#
#
# Revision history:
#
# 2007-01-xx created by weizen_42 at ipcop-forum dot de
#
# 2007-01-13  added config option: https port for loginpage
#

use strict;

# enable only the following on debugging purpose
use warnings;
use CGI::Carp 'fatalsToBrowser';

use LWP::UserAgent;

require '/var/ipcop/general-functions.pl';
require "${General::swroot}/lang.pl";
require "${General::swroot}/header.pl";

my $debug = 0;
my $version = 'v0.1.0';
# checking for new version
my $addonname = 'copspot';
my $onlineversion = '';
my $onlinelink = '';
my $timestamplastcheck = '/var/ipcop/copspot/lastcheck';
my $flagdonotcheck = '/var/ipcop/copspot/noversioncheck';

# fetch current CopSpot secure port
my $https = `/bin/grep Listen /var/ipcop/copspot/apache.conf | cut -f 2 -d ' '`;
chomp($https);

my %copspot_settings = ();
my %netsettings = ();
my %hashxtiface = ();
# blue, gray1, gray2, ...
my @ifaces = ();
# iface IP is not needed at the moment, but leave just in case
my %ifaces_ip = ();
# ethx device we are running on
my %ifaces_dev = ();

my %checked = ();
my %selected = ();
my $errormessage = '';
my $i = 0;

&General::readhash("${General::swroot}/ethernet/settings", \%netsettings);
if ( $netsettings{'BLUE_DEV'} ne '' )
{
  # blue interface available
  push(@ifaces, "blue");
  $ifaces_ip{'blue'} = $netsettings{'BLUE_ADDRESS'};
  $ifaces_dev{'blue'} = $netsettings{'BLUE_DEV'};
}

if ( -e "${General::swroot}/xtiface/xtiface.conf" )
{
  &General::readhash("${General::swroot}/xtiface/xtiface.conf", \%hashxtiface);
  for $i ( 1 .. 4 ) 
  {
    if ( ($hashxtiface{"GRAY${i}_DEV"} ne '') && ($hashxtiface{"GRAY${i}_DEV"} ne 'disabled') )
    {
      # gray interface available
      push(@ifaces, "gray$i");
      $ifaces_ip{"gray$i"} = $hashxtiface{"GRAY${i}_ADDRESS"};
      $ifaces_dev{"gray$i"} = $hashxtiface{"GRAY${i}_DEV"};
    }
  }
}

$copspot_settings{'ACTION'} = '';
$copspot_settings{'ENABLE'} = '';
$copspot_settings{'IFACE'} = '';
$copspot_settings{'HTTPSPORT'} = 55445;
$copspot_settings{'NET'} = '192.168.182.0/24';
$copspot_settings{'RADIUSSERVER1'} = '192.168.1.1';
$copspot_settings{'RADIUSSERVER2'} = '';
$copspot_settings{'RADIUSAUTHPORT'} = 1812;
$copspot_settings{'RADIUSACCTPORT'} = 1813;
$copspot_settings{'RADIUSSECRET'} = 'testing123';
$copspot_settings{'UAMNAME'} = 'CopSpot';
$copspot_settings{'UAMSECRET'} = 'theuamsecret';
$copspot_settings{'UAMPORT'} = 3990;
$copspot_settings{'UAMDOMAINS'} = 'www.google.com,www.ipcop.org';

# will be calculated / set when saving
$copspot_settings{'CHILLIIP'} = '';               

# get rid of used only once warnings
my @onlyonce = ( $Header::colourgreen, $Header::colourred );
undef @onlyonce;

&Header::showhttpheaders();
&Header::getcgihash(\%copspot_settings);

&Header::openpage("CopSpot", 1, '');
&Header::openbigbox('100%', 'left');


if ( $ifaces[0] eq '' )
{
  $errormessage = $Lang::tr{'copspot error no interface'};
  $copspot_settings{'ENABLE'} = 'off';
}


if ( $copspot_settings{'ACTION'} eq 'restart' )
{
  system("/usr/local/bin/copspotctrl restart >/dev/null");
}

if ( ($errormessage eq '') && ($copspot_settings{'ACTION'} eq 'save') )
{
  if ( $copspot_settings{"UAMDOMAINS"} ne '' )
  {
    $copspot_settings{"UAMDOMAINS"} =~ s/\r\n/,/gm
  }

  # verify network
  unless( &General::validipandmask($copspot_settings{'NET'}) )
  {
    $errormessage = $Lang::tr{'copspot error net'};
    goto ERROR;
  }
  # verify https port
	unless( &General::validport($copspot_settings{'HTTPSPORT'}) )
	{
		$errormessage = $Lang::tr{'copspot error httpsport'};
		goto ERROR;
	}
	unless ( $copspot_settings{'HTTPSPORT'} > 445 )
	{
		$errormessage = $Lang::tr{'copspot error httpsport'};
		goto ERROR;
	}
  # verify Radiusserver1
  unless( &General::validip($copspot_settings{'RADIUSSERVER1'}) )
  {
    $errormessage = $Lang::tr{'copspot error radiusserver1'};
    goto ERROR;
  }
  # verify Radiusserver2
  unless( ($copspot_settings{'RADIUSSERVER2'} eq '') || &General::validip($copspot_settings{'RADIUSSERVER2'}) )
  {
    $errormessage = $Lang::tr{'copspot error radiusserver2'};
    goto ERROR;
  }
  # verify Authport
	unless( &General::validport($copspot_settings{'RADIUSAUTHPORT'}) )
	{
		$errormessage = $Lang::tr{'copspot error authport'};
		goto ERROR;
	}
  # verify Acctport
	unless( &General::validport($copspot_settings{'RADIUSACCTPORT'}) )
	{
		$errormessage = $Lang::tr{'copspot error acctport'};
		goto ERROR;
	}
  # verify UAM Listenport
	unless( &General::validport($copspot_settings{'UAMPORT'}) )
	{
		$errormessage = $Lang::tr{'copspot error uamport'};
		goto ERROR;
	}
	unless ( $copspot_settings{'HTTPSPORT'} ne $copspot_settings{'UAMPORT'} )
	{
		$errormessage = $Lang::tr{'copspot error ports'};
		goto ERROR;
	}

  if ( $copspot_settings{'ENABLE'} eq 'on' )
  {
    $copspot_settings{'NET'} =~ /^(\d+)\.(\d+)\.(\d+)\.(\d+)/;
    my $ip4 = $4+1;
    $copspot_settings{'CHILLIIP'} = $1 . "." . $2 . "." . $3 . "." . $ip4;
    $copspot_settings{'DEVICE'} = $ifaces_dev{"$copspot_settings{'IFACE'}"};
  }

  &General::writehash("${General::swroot}/copspot/settings", \%copspot_settings);  
  &writeconfig;

ERROR:
}


if ( $errormessage )
{
  &Header::openbox('100%', 'left', $Lang::tr{'error messages'});
  print "<class name='base'>$errormessage</class>";
  &Header::closebox();
}


# Found this usefull piece of code in BlockOutTraffic AddOn  8-)
#   fwrules.cgi
###############
# DEBUG DEBUG
if ( $debug )
{
&Header::openbox('100%', 'left', 'DEBUG');
my $debugCount = 0;
foreach my $line (sort keys %copspot_settings) {
  print "$line = $copspot_settings{$line}<br />\n";
  $debugCount++;
}
print "&nbsp;Count: $debugCount\n";
&Header::closebox();
}
# DEBUG DEBUG
###############


#
# Check for new version
#
&checkfornewversion($addonname, $version);
if ( $onlineversion ne '' )
{
  &Header::openbox('100%', 'left', $Lang::tr{'info'});
  print <<END
<table width="100%"><tr>
<td>$Lang::tr{'copspot newversion'} <a href="$onlinelink" target="_blank"><b>$onlineversion</b></a></td>
</tr></table>
END
;
  &Header::closebox();
}

if ( $errormessage eq '' ) 
{
  if ( -e "${General::swroot}/copspot/settings" )
  {
    &General::readhash("${General::swroot}/copspot/settings", \%copspot_settings);
  }
}

my $chilli_ps = `/bin/ps -A | /bin/grep chilli`;
my $copspot_status = 'CopSpot is not running';
my $bgcolor_status = "style='background-color:${Header::colourred}'";

if ( $chilli_ps ne '' )
{
  $copspot_status = 'CopSpot is running';
  $bgcolor_status = "style='background-color:${Header::colourgreen}'";
}


$checked{'ENABLE'}{'off'} = '';
$checked{'ENABLE'}{'on'} = '';
$checked{'ENABLE'}{$copspot_settings{'ENABLE'}} = "checked='checked'";
foreach ( @ifaces ) 
{
  $selected{'IFACE'}{$_} = '';
}
$selected{'IFACE'}{$copspot_settings{'IFACE'}} = "selected='selected'";


if ( $copspot_settings{"UAMDOMAINS"} ne '' )
{
  $copspot_settings{"UAMDOMAINS"} =~ s/,/\n/gm
}

##########
#
# Box with config / settings
#
##########
&Header::openbox('100%', 'left', "$Lang::tr{'status'}");
print <<END
<form method='post' name='form1' action='$ENV{'SCRIPT_NAME'}'>
<table width='100%'>
<tr>
	<td width='25%' valign='center'><b>$copspot_status</b></td>
  <td width='25%' valign='center'><input type='submit' name='SUBMIT' value=$Lang::tr{'restart'} /><input type='hidden' name='ACTION' value='restart' /></td>
	<td align='right' valign='top'><a href='http://www.chillispot.org/' target='_blank'><img src='http://www.ipadd.de/stuff/copspot4.png'></a></td>
</tr>
</table>
</form>
END
;
&Header::closebox(); 

&Header::openbox('100%', 'left', "$Lang::tr{'settings'}");

print <<END
<form method='post' name='form2' action='$ENV{'SCRIPT_NAME'}'>
<table width='100%'>
<tr>
  <td width='25%' class='base'>$Lang::tr{'enabled'}</td>
  <td width='20%' class='base'><input type='checkbox' name='ENABLE' $checked{'ENABLE'}{'on'} /></td>
  <td width='25%' class='base'>&nbsp;</td>
  <td width='30%' class='base'>&nbsp;</td>
</tr><tr>
  <td class='base'>$Lang::tr{'interface'}:</td>
  <td class='base'><select name='IFACE'>
END
;
foreach ( @ifaces ) 
{
  print "<option value='$_' $selected{'IFACE'}{$_}>$Lang::tr{$_}</option>";
}
print <<END
  </select></td>
  <td class='base' colspan='2'>&nbsp;</td>
</tr><tr>
  <td class='base'>$Lang::tr{'copspot network'}:</td>
  <td class='base' colspan='3'><input type='text' name='NET' value='$copspot_settings{"NET"}' size='18' /></td>
</tr><tr>
  <td class='base'>$Lang::tr{'copspot https'}:</td>
  <td class='base'><input type='text' name='HTTPSPORT' value='$copspot_settings{"HTTPSPORT"}' size='5' /></td>
  <td class='base' colspan='2'> $Lang::tr{'current'}: $https</td>
</tr><tr><td class='base' colspan='4'><hr /></td></tr><tr>
  <td class='base' colspan='4'><b>$Lang::tr{'copspot radius settings'}</b></td>
</tr><tr>
  <td class='base'>$Lang::tr{'copspot radius server'} 1:</td>
  <td class='base'><input type='text' name='RADIUSSERVER1' value='$copspot_settings{"RADIUSSERVER1"}' size='18' /></td>
  <td class='base'>$Lang::tr{'copspot radius authport'}:</td>
  <td class='base'><input type='text' name='RADIUSAUTHPORT' value='$copspot_settings{"RADIUSAUTHPORT"}' size='5' /></td>
</tr><tr>
  <td class='base'>$Lang::tr{'copspot radius server'} 2:&nbsp;<img src='/blob.gif' alt='*' /></td>
  <td class='base'><input type='text' name='RADIUSSERVER2' value='$copspot_settings{"RADIUSSERVER2"}' size='18' /></td>
  <td class='base'>$Lang::tr{'copspot radius acctport'}:</td>
  <td class='base'><input type='text' name='RADIUSACCTPORT' value='$copspot_settings{"RADIUSACCTPORT"}' size='5' /></td>
</tr><tr>
  <td class='base'>$Lang::tr{'copspot radius secret'}:</td>
  <td class='base' colspan='4'><input type='password' name='RADIUSSECRET' value='$copspot_settings{"RADIUSSECRET"}' size='40' /></td>
</tr><tr><td class='base' colspan='4'><hr /></td></tr><tr>
  <td class='base' colspan='4'><b>$Lang::tr{'copspot UAM settings'}</b></td>
</tr><tr>
  <td class='base'>$Lang::tr{'copspot UAM name'}:</td>
  <td class='base' colspan='3'><input type='text' name='UAMNAME' value='$copspot_settings{"UAMNAME"}' size='40' /></td>
</tr><tr>
  <td class='base'>$Lang::tr{'copspot UAM secret'}:</td>
  <td class='base' colspan='3'><input type='password' name='UAMSECRET' value='$copspot_settings{"UAMSECRET"}' size='40' /></td>
</tr><tr>
  <td class='base'>$Lang::tr{'copspot UAM port'}:</td>
  <td class='base' colspan='3'><input type='text' name='UAMPORT' value='$copspot_settings{"UAMPORT"}' size='5' /></td>
</tr><tr>
  <td class='base' colspan='4'>$Lang::tr{'copspot UAM domains'}:</td>
</tr><tr>
  <td class='base'>&nbsp;</td>
  <td class='base' colspan='3'><textarea name='UAMDOMAINS' cols='32' rows='5' wrap='off'>$copspot_settings{"UAMDOMAINS"}</textarea></td>
</tr>
</table>
<hr />
<table width='100%'>
<tr>
  <td width='70%' class='base' valign='top'><img src='/blob.gif' alt='*' />&nbsp;$Lang::tr{'this field may be blank'}</td>
  <td width='30%'><input type='submit' value='$Lang::tr{'save'}' /><input type='hidden' name='ACTION' value='save' /></td>
</tr>
</table>
</form>
END
;


print <<END
<hr />
<table width='100%'>
<tr><td>&nbsp;</td><td align='right'><b><small><a href="http://www.chillispot.org" target="_blank">Chillispot v1.1</a></small></b></td></tr>
<tr><td>&nbsp;</td><td align='right'><b><small><a href="http://www.ban-solms.de/t/IPCop.html" target="_blank">CopSpot Addon $version</a></small></b></td></tr>
</table>
</form>
END
;

&Header::closebox(); 


&Header::closebigbox();
&Header::closepage();

if ( ($errormessage eq '') && ($copspot_settings{'ACTION'} eq 'save') && ($copspot_settings{'HTTPSPORT'} ne $https) ) 
{
  &writeconfigapache;
}


sub writeconfig
{
  $copspot_settings{'RADIUSSERVER2'} = $copspot_settings{'RADIUSSERVER1'} if ( $copspot_settings{'RADIUSSERVER2'} eq '' );

  open(FILE, ">${General::swroot}/copspot/chilli.conf");
  flock(FILE, 2);
print FILE <<END
net $copspot_settings{'NET'}
dns1 $copspot_settings{'CHILLIIP'}
ipup /usr/local/sbin/chilli.ipup
radiusserver1 $copspot_settings{'RADIUSSERVER1'}
radiusserver2 $copspot_settings{'RADIUSSERVER2'}
radiusauthport $copspot_settings{'RADIUSAUTHPORT'}
radiusacctport $copspot_settings{'RADIUSACCTPORT'}
radiussecret   $copspot_settings{'RADIUSSECRET'}
dhcpif $copspot_settings{'DEVICE'}
uamallowed  $copspot_settings{'CHILLIIP'}
uamserver   https://$copspot_settings{'CHILLIIP'}:$copspot_settings{'HTTPSPORT'}/copspotlogin.cgi
uamsecret   $copspot_settings{'UAMSECRET'}
uamport     $copspot_settings{'UAMPORT'}
END
;
  print FILE "uamallowed $copspot_settings{'UAMDOMAINS'}\n" if ( $copspot_settings{'UAMDOMAINS'} ne '' );
  close FILE;

  open(FILE, ">${General::swroot}/copspot/uamname.pl");
  flock(FILE, 2);
  print FILE "\$uamname = '$copspot_settings{'UAMNAME'}';\n";
  close(FILE);

  open(FILE, ">${General::swroot}/copspot/uamsecret.pl");
  flock(FILE, 2);
  print FILE "\$uamsecret = '$copspot_settings{'UAMSECRET'}';\n";
  close(FILE);

  system("/usr/local/bin/copspotctrl restart >/dev/null");
}

sub writeconfigapache
{
  # might try to listen to CHILLIIP:HTTPSPORT
  # but if Chilli was not started for some reason we also loose Apache (tun + CHILLIIP do not exist) -> not good
  my $blub = "/bin/sed -i -e 's+Listen.*\$+Listen $copspot_settings{'HTTPSPORT'}+' -e 's+<VirtualHost _default_.*\$+<VirtualHost _default_:$copspot_settings{'HTTPSPORT'}>+' /var/ipcop/copspot/apache.conf";
#  system("echo \"$blub\" > /tmp/blub");

  system($blub);
  system("/usr/local/bin/copspotctrl apache >/dev/null");
}

sub checkfornewversion
{
  my $addon = shift;
  my $version = shift;
  $onlineversion = '';

  if ( -e $flagdonotcheck )
  {
    return;
  }

  # only check if we are online and last check was some time ago
  if ( (! -e '/var/ipcop/red/active') || (-e $timestamplastcheck) && (int(-M $timestamplastcheck) < 5) )
  {
    return;
  }

#workaround to suppress a warning when a variable is used only once
  my @dummy = ( $General::version );
  undef (@dummy);

  my $ua = LWP::UserAgent->new;
  $ua->timeout(120);
  $ua->agent("Mozilla/4.0 (compatible; IPCop $General::version; $version)");
  my $content = $ua->get("http://ipcop-addons.ath.cx/version/$addon");

  if ( $content->is_success )
  {
    # compare the versions, format is v1.2.3
    $content->content =~ /v(\d+).(\d+).(\d+)/;
    my $ver1 = $1;
    my $ver2 = $2;
    my $ver3 = $3;

    $version =~ /v(\d+).(\d+).(\d+)/;

    if ( ($ver1 > $1) || (($ver1 == $1) && ($ver2 > $2)) || (($ver1 == $1) && ($ver2 == $2) && ($ver3 > $3)) )
    {
      $onlineversion = "v$ver1.$ver2.$ver3";

      $content->content =~ /http(.*)/;
      $onlinelink = "http$1";
    }
    else
    {
      # no news, recheck in a couple of days
      system("touch $timestamplastcheck");
    }
  }
}
